@@ -21,6 +21,8 @@ import Database.HDBC.Locale (defaultTimeLocale, iso8601DateFormat)
 import Data.Ratio
 import Data.Convertible
 import Data.Fixed
+import qualified Data.Text as TS
+import qualified Data.Text.Lazy as TL
 
 quickError :: (Typeable a, Convertible SqlValue a) => SqlValue -> ConvertResult a
 quickError sv = convError "incompatible types" sv
@@ -240,6 +242,9 @@ instance Eq SqlValue where
     a == b = ((safeFromSql a)::ConvertResult String) == 
              ((safeFromSql b)::ConvertResult String)
 
+instance Convertible SqlValue SqlValue where
+    safeConvert = return
+
 instance Convertible String SqlValue where
     safeConvert = return . SqlString
 instance Convertible SqlValue String where
@@ -277,6 +282,18 @@ instance Convertible SqlValue String where
     safeConvert (SqlTimeDiff x) = return . show $ x
     safeConvert y@(SqlNull) = quickError y
 
+instance Convertible TS.Text SqlValue where
+    safeConvert = return . SqlString . TS.unpack
+
+instance Convertible SqlValue TS.Text where
+    safeConvert = fmap TS.pack . safeConvert
+
+instance Convertible TL.Text SqlValue where
+    safeConvert = return . SqlString . TL.unpack
+
+instance Convertible SqlValue TL.Text where
+    safeConvert = fmap TL.pack . safeConvert
+
 #ifdef __HUGS__
 instance Typeable B.ByteString where
     typeOf _ = mkTypeName "ByteString"
@@ -489,7 +506,7 @@ instance Convertible SqlValue Bool where
     safeConvert (SqlTimeDiff x) = numToBool x
     safeConvert y@(SqlNull) = quickError y
 
-numToBool :: Num a => a -> ConvertResult Bool
+numToBool :: (Eq a, Num a) => a -> ConvertResult Bool
 numToBool x = Right (x /= 0)
 
 instance Convertible Char SqlValue where
@@ -497,10 +514,8 @@ instance Convertible Char SqlValue where
 instance Convertible SqlValue Char where
     safeConvert (SqlString [x]) = return x
     safeConvert y@(SqlString _) = convError "String length /= 1" y
-    safeConvert y@(SqlByteString x) = 
-        case B.length x of
-          1 -> safeConvert . SqlString . BUTF8.toString $ x
-          _ -> convError "ByteString length /= 1" y
+    safeConvert (SqlByteString x) =
+          safeConvert . SqlString . BUTF8.toString $ x
     safeConvert y@(SqlInt32 _) = quickError y
     safeConvert y@(SqlInt64 _) = quickError y
     safeConvert y@(SqlWord32 _) = quickError y
